/* GIMP Plugin Preview Widget                                                 
 * Copyright (C) 1998-1999 Shawn T. Amundson                
 * Modifications (C) 1999-2003 Ernst Lippe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.           
 *                                                                            
 * This library is distributed in the hope that it will be useful,             
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Version $Id: gimppreview.h,v 1.7 2003/06/11 11:09:09 el Exp $
 */

#ifndef __GIMP_PREVIEW_H_INCLUDED
#define __GIMP_PREVIEW_H_INCLUDED

#include <gtk/gtk.h>
#include <libgimp/gimp.h>
#include "gimppreviewimage.h"

G_BEGIN_DECLS
#define GIMP_TYPE_PREVIEW            (gimp_preview_get_type ())
#define GIMP_PREVIEW(obj)            (GTK_CHECK_CAST ((obj), GIMP_TYPE_PREVIEW, GimpPreview))
#define GIMP_PREVIEW_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GIMP_TYPE_PREVIEW, GimpPreviewClass))
#define GIMP_IS_PREVIEW(obj)         (GTK_CHECK_TYPE ((obj), GIMP_TYPE_PREVIEW))
#define GIMP_IS_PREVIEW_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GIMP_TYPE_PREVIEW))
typedef struct _GimpPreview GimpPreview;
typedef struct _GimpPreviewClass GimpPreviewClass;
typedef struct _GimpPreviewUpdateEvent GimpPreviewUpdateEvent;

struct _GimpPreviewClass
{
  GtkTableClass parent_class;

  void (*update_preview) (GimpPreview * preview,
                          GimpPreviewUpdateEvent * event);
};

/* All data in the GimpPreview is private */
struct _GimpPreview
{
  /*< private > */
  GtkTable parent;

  GtkWidget *label;
  GtkWidget *button_minus;
  GtkWidget *button_plus;
  GtkWidget *zoom_controls;
  GtkWidget *event_box;
  GimpPreviewImage *image;
  GtkWidget *progress_bar;
  GtkWidget *hscroll_bar, *vscroll_bar;

  gint n_scales;                /* Number of differents scale factor in scales */
  gdouble *scales;              /* Allowed scale factors. Must be sorted ascending */

  gboolean in_drag;             /* Is true while the preview is dragged     */
  gint drag_x;                  /* Mouse x-position when dragging started   */
  gint drag_y;                  /* Mouse y-position when dragging started   */
  gint orig_preview_x;          /* Value of preview_x when dragging started */
  gint orig_preview_y;          /* Value of preview_y when dragging started */
  /* Time of the previous call to the GLib main loop in gimp_preview_process_events */
  GTimeVal previous_event_time;
};

/**
 * GimpPreviewUpdateEvent:
 * @scale: Current scale of the preview.
 * @image_x: Left of requested unscaled data in #image-coordinates. 
 * @image_y: Top of requested unscaled data in #image-coordinates.
 * @image_width: Width of requested unscaled data in #image-coordinates.
 * @image_height: Height of requested unscaled data in #image-coordinates.
 * @preview_x: Left of preview in #preview-coordinates.
 * @preview_y: Top of preview in #preview-coordinates.
 * @preview_width: Width of the preview.
 * @preview_height: Height of the preview.
 *
 * This event will be sent with the "#update-preview" signal,
 * that is emitted by the preview when the rendering function
 * should render a new image for the preview.
 **/
struct _GimpPreviewUpdateEvent
{
  gdouble scale;                /* Scale of preview */

  /* Left/Top of requested unscaled data in image coordinates */
  gint image_x;
  gint image_y;

  /* Width/Height of requested unscaled data in image coordinates */
  gint image_width;
  gint image_height;

  /* Left/Top of preview in preview coordinates */
  gint preview_x;
  gint preview_y;

  /* Width/Height of the preview */
  gint preview_width;
  gint preview_height;
};

GtkType gimp_preview_get_type (void);

GtkWidget *gimp_preview_new (GimpDrawable * drawable);
GtkWidget *gimp_preview_new_with_args (GimpDrawable * drawable,
                                       gint preview_width,
                                       gint preview_height,
                                       gdouble initial_scale,
                                       gboolean show_progressbar,
                                       gboolean show_zoom_controls,
                                       gboolean show_scroll_bars);

gboolean gimp_preview_draw_scaled_drawable (GimpPreview * preview,
                                            GimpDrawable * drawable);

gboolean gimp_preview_draw_unscaled_drawable (GimpPreview * preview,
                                              GimpDrawable * drawable);

gboolean gimp_preview_draw_scaled_row (GimpPreview * preview,
                                       GimpImageType type,
                                       gint row,
                                       gint width, const guchar * data);

gboolean gimp_preview_draw_unscaled_row (GimpPreview * preview,
                                         GimpImageType type,
                                         gint row,
                                         gint width, const guchar * data);

gboolean gimp_preview_progress_set_fraction (GimpPreview * preview,
                                             gdouble fraction);

void gimp_preview_update (GimpPreview * preview);

void gimp_preview_set_position (GimpPreview * preview,
                                gint x_pos, gint y_pos);
void gimp_preview_get_preview_position (GimpPreview * preview,
                                        gint * prev_x_pos, gint * prev_y_pos,
                                        gint * prev_width,
                                        gint * prev_height);
void gimp_preview_get_image_position (GimpPreview * preview, gint * im_x_pos,
                                      gint * im_y_pos, gint * im_width,
                                      gint * im_height);

void gimp_preview_set_scale (GimpPreview * preview, gdouble scale);
gdouble gimp_preview_get_scale (GimpPreview * preview);

gboolean gimp_preview_get_progress_bar_visible (GimpPreview * preview);
void gimp_preview_set_progress_bar_visible (GimpPreview * preview,
                                            gboolean visible);

gboolean gimp_preview_get_zoom_controls_visible (GimpPreview * preview);
void gimp_preview_set_zoom_controls_visible (GimpPreview * preview,
                                             gboolean visible);
gboolean gimp_preview_get_scroll_bars_visible (GimpPreview * preview);
void gimp_preview_set_scroll_bars_visible (GimpPreview * preview,
                                           gboolean visible);

gboolean gimp_preview_get_show_original (GimpPreview * preview);
void gimp_preview_set_show_original (GimpPreview * preview,
                                     gboolean show_original);
void gimp_preview_set_scales (GimpPreview * preview, gint n_scales,
                              const gdouble * scales);


GtkAdjustment *gimp_preview_get_x_adjustment (GimpPreview * preview);
GtkAdjustment *gimp_preview_get_y_adjustment (GimpPreview * preview);
GtkAdjustment *gimp_preview_get_scale_adjustment (GimpPreview * preview);

G_END_DECLS
#endif /* __GIMP_PREVIEW_H__ */
