/* The GIMP -- an image manipulation program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * gimppreviewdialog.c
 * Copyright (C) 2002 Maurits Rijk <lpeek.mrijk@consunet.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* TODO:
 *
 * - Handle zoom in/out
 * - Implement paner
 * - Implement Reset
 * - Parameter Settings shouldn't always fill up complete frame
 * - Cleanup :)
 *
 */

#include <stdio.h>
#include <string.h>

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#include "gimppreview.h"
#include "gimppreviewdialog.h"

#define PREVIEW_SIZE 128

#ifndef lint
static char vcid[] G_GNUC_UNUSED =
  "$Id: gimppreviewdialog.c,v 1.2 2003/06/11 11:09:08 el Exp $";
#endif /* lint */

static gboolean _run = FALSE;

static void
dialog_ok_callback (GtkWidget * widget, gpointer data)
{
  _run = TRUE;

  gtk_widget_destroy (GTK_WIDGET (data));
}

/* Fix me: quick hack! */
static GtkWidget *preview;

void
preview_update_effect (void)
{
  gimp_preview_update (GIMP_PREVIEW (preview));
}

gboolean
do_preview_dialog (GimpDrawable * drawable, EffectParam_t * param)
{
  GtkWidget *dlg;
  GtkWidget *hbox;
  GtkWidget *preview_orig;
  GtkWidget *frame;
  GtkWidget *settings;

  gimp_ui_init ("preview", FALSE);

  dlg = gimp_dialog_new (param->title, param->wmclass_name,
                         gimp_standard_help_func, param->help_data,
                         GTK_WIN_POS_MOUSE,
                         FALSE, TRUE, FALSE,
                         GTK_STOCK_CANCEL, gtk_widget_destroy,
                         NULL, 1, NULL, FALSE, TRUE,
                         GTK_STOCK_OK, dialog_ok_callback,
                         NULL, NULL, NULL, TRUE, FALSE, NULL);

  g_signal_connect (G_OBJECT (dlg), "destroy",
                    G_CALLBACK (gtk_main_quit), NULL);

  /* Horizontal box to hold previews and preview actions */
  hbox = gtk_hbox_new (FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), hbox);
  gtk_widget_show (hbox);

  /* Preview widget that holds source image */
  preview_orig = gimp_preview_new_with_args (drawable, PREVIEW_SIZE,
                                             PREVIEW_SIZE, 1.0, TRUE, TRUE,
                                             TRUE);
  gtk_container_add (GTK_CONTAINER (hbox), preview_orig);
  g_signal_connect (G_OBJECT (preview_orig), "update-preview",
                    G_CALLBACK (param->effect_func), drawable);

  gtk_widget_show (preview_orig);

  preview = preview_orig;
#ifdef _NOT_USED_YET_
  /* Table to hold zoom buttons */
  actions = gtk_table_new (3, 3, FALSE);
  gtk_table_set_homogeneous (GTK_TABLE (actions), FALSE);
  gtk_container_add (GTK_CONTAINER (hbox), actions);
  gtk_widget_show (actions);

  /* Zoom in button */
  image = gtk_image_new_from_stock (GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image);
  zoom_in = gtk_button_new ();
  gtk_container_add (GTK_CONTAINER (zoom_in), image);
  gtk_table_attach (GTK_TABLE (actions), zoom_in, 0, 1, 1, 2, GTK_SHRINK,
                    GTK_FILL, 0, 0);
  g_signal_connect (G_OBJECT (zoom_in), "clicked",
                    G_CALLBACK (zoom_in_cb), NULL);
  gtk_widget_show (zoom_in);

  /* Paner button */
  image = gtk_image_new_from_stock (GTK_STOCK_NEW, GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image);
  paner = gtk_button_new ();
  gtk_container_add (GTK_CONTAINER (paner), image);
  gtk_table_attach (GTK_TABLE (actions), paner, 1, 2, 1, 2, GTK_SHRINK,
                    GTK_FILL, 0, 0);
  g_signal_connect (G_OBJECT (paner), "clicked", G_CALLBACK (paner_cb), NULL);
  gtk_widget_show (paner);

  /* Zoom out button */
  image = gtk_image_new_from_stock (GTK_STOCK_ZOOM_OUT, GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image);
  zoom_out = gtk_button_new ();
  gtk_container_add (GTK_CONTAINER (zoom_out), image);
  gtk_table_attach (GTK_TABLE (actions), zoom_out, 2, 3, 1, 2, GTK_SHRINK,
                    GTK_SHRINK, 0, 0);
  g_signal_connect (G_OBJECT (zoom_out), "clicked",
                    G_CALLBACK (zoom_out_cb), NULL);
  gtk_widget_show (zoom_out);

  /* Preview widget that holds image after effect */
  preview = gimp_effect_preview_new (drawable, PREVIEW_SIZE, PREVIEW_SIZE);
  gtk_container_add (GTK_CONTAINER (hbox), preview);
  gimp_effect_preview_set_effect_func (GIMP_PREVIEW (preview),
                                       param->effect_func);
  gtk_widget_show (preview);
#endif
  /* Frame to hold settings */
  frame = gtk_frame_new ("Parameter Settings");
  gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), frame);
  gtk_widget_show (frame);

  settings = param->get_settings_widget (frame, param->settings_data);
  gtk_container_add (GTK_CONTAINER (frame), settings);
  gtk_widget_show (settings);

  gtk_widget_show (dlg);

  gtk_main ();
  gdk_flush ();

  return _run;
}
