/* GIMP Plugin Preview Widget                                                 
 * Copyright (C) 1998-1999 Shawn T. Amundson                
 * Modifications (C) 1999-2003 Ernst Lippe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.           
 *                                                                            
 * This library is distributed in the hope that it will be useful,             
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Version $Id: gimppreviewimage.h,v 1.6 2003/06/11 11:09:11 el Exp $
 */

#ifndef _GIMP_PREVIEW_IMAGE_H_INCLUDED
#define _GIMP_PREVIEW_IMAGE_H_INCLUDED

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

G_BEGIN_DECLS
#define GIMP_TYPE_PREVIEW_IMAGE            (gimp_preview_image_get_type ())
#define GIMP_PREVIEW_IMAGE(obj)            (GTK_CHECK_CAST ((obj), GIMP_TYPE_PREVIEW_IMAGE, GimpPreviewImage))
#define GIMP_PREVIEW_IMAGE_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GIMP_TYPE_PREVIEW_IMAGE, GimpPreviewImageClass))
#define GIMP_IS_PREVIEW_IMAGE(obj)         (GTK_CHECK_TYPE ((obj), GIMP_TYPE_PREVIEW_IMAGE))
#define GIMP_IS_PREVIEW_IMAGE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GIMP_TYPE_PREVIEW_IMAGE))
typedef struct _GimpPreviewImage GimpPreviewImage;
typedef struct _GimpPreviewImageClass GimpPreviewImageClass;
typedef struct _GimpPreviewImageUpdateEvent GimpPreviewImageUpdateEvent;

struct _GimpPreviewImageClass
{
  GtkImageClass parent_class;

  void (*update_preview) (GimpPreviewImage * preview_image,
                          GimpPreviewImageUpdateEvent * event);
};

/*< private >*/
typedef enum
{
  GPI_RENDER_OK,                /* No update needed */
  GPI_RENDER_UPDATE_NEEDED,     /* New image must be rendered */
  GPI_RENDER_RENDERING          /* Busy rendering new image */
}
GimpPreviewImageRenderStateType;

/* All data in the GimpPreviewImage is private */
struct _GimpPreviewImage
{
  /*< private >*/
  GtkImage parent;

  GimpDrawable *drawable;       /* The drawable that is shown */
  gint drawable_has_alpha;      /* Does the drawable have alpha */
  gint width;                   /* Width of Preview   */
  gint height;                  /* Height of Preview  */
  gint requested_width;
  gint requested_height;
  GtkAdjustment *scale_adjustment;
  GtkAdjustment *x_adjustment;
  GtkAdjustment *y_adjustment;
  gboolean show_original;       /* Show original image while scrolling */
  gdouble previous_scale;       /* Is used to recompute the new position */
  gboolean idle_fun_busy;       /* This is a semaphore for gimp_preview_image_update_preview_idle_fun() */

  /* Indicate if plug-in should render a new image */
  GimpPreviewImageRenderStateType render_state;
};

/**
 * GimpPreviewImageUpdateEvent:
 * @scale: Current scale of the preview.
 * @image_x: Left of requested unscaled data in #image-coordinates. 
 * @image_y: Top of requested unscaled data in #image-coordinates.
 * @image_width: Width of requested unscaled data in #image-coordinates.
 * @image_height: Height of requested unscaled data in #image-coordinates.
 * @preview_x: Left of preview in #preview-coordinates.
 * @preview_y: Top of preview in #preview-coordinates.
 * @preview_width: Width of the preview.
 * @preview_height: Height of the preview.
 *
 * This event will be sent with the "#update-preview" signal,
 * that is emitted by the preview when the rendering function
 * should render a new image for the preview.
 **/
struct _GimpPreviewImageUpdateEvent
{
  gdouble scale;                /* Scale of preview */

  /* Left/Top of requested unscaled data in image coordinates */
  gint image_x;
  gint image_y;

  /* Width/Height of requested unscaled data in image coordinates */
  gint image_width;
  gint image_height;

  /* Left/Top of preview in preview coordinates */
  gint preview_x;
  gint preview_y;

  /* Width/Height of the preview */
  gint preview_width;
  gint preview_height;
};

GtkType gimp_preview_image_get_type (void);

GtkWidget *gimp_preview_image_new (GimpDrawable * drawable);
GtkWidget *gimp_preview_image_new_with_args (GimpDrawable * drawable,
                                             gint preview_width,
                                             gint preview_height,
                                             gdouble initial_scale);

gboolean gimp_preview_image_draw_scaled_drawable (GimpPreviewImage *
                                                  preview_image,
                                                  GimpDrawable * drawable);

gboolean gimp_preview_image_draw_unscaled_drawable (GimpPreviewImage *
                                                    preview_image,
                                                    GimpDrawable * drawable);

gboolean gimp_preview_image_draw_scaled_row (GimpPreviewImage * preview_image,
                                             GimpImageType type,
                                             gint row,
                                             gint width, const guchar * data);

gboolean gimp_preview_image_draw_unscaled_row (GimpPreviewImage *
                                               preview_image,
                                               GimpImageType type,
                                               gint row,
                                               gint width,
                                               const guchar * data);

void gimp_preview_image_update (GimpPreviewImage * preview_image);

void gimp_preview_image_set_position (GimpPreviewImage * preview_image,
                                      gint x_pos, gint y_pos);
void gimp_preview_image_get_preview_position (GimpPreviewImage *
                                              preview_image,
                                              gint * prev_x_pos,
                                              gint * prev_y_pos,
                                              gint * prev_width,
                                              gint * prev_height);
void gimp_preview_image_get_image_position (GimpPreviewImage * preview_image,
                                            gint * im_x_pos, gint * im_y_pos,
                                            gint * im_width,
                                            gint * im_height);

void gimp_preview_image_set_scale (GimpPreviewImage * preview_image,
                                   gdouble scale);
gdouble gimp_preview_image_get_scale (GimpPreviewImage * preview_image);

void gimp_preview_image_set_show_original (GimpPreviewImage * preview_image,
                                           gboolean show_original);
gboolean gimp_preview_image_get_show_original (GimpPreviewImage *
                                               preview_image);

gboolean gimp_preview_image_is_drawing_allowed (GimpPreviewImage *
                                                preview_image);

void gtk_adjustment_set_correct_value (GtkAdjustment * adjustment,
                                       gdouble value);

GtkAdjustment *gimp_preview_image_get_x_adjustment (GimpPreviewImage *
                                                    preview_image);
GtkAdjustment *gimp_preview_image_get_y_adjustment (GimpPreviewImage *
                                                    preview_image);
GtkAdjustment *gimp_preview_image_get_scale_adjustment (GimpPreviewImage *
                                                        preview_image);

G_END_DECLS
#endif
