/* GIMP Plugin Preview Widget                                                 
 * Copyright (C) 2003 Ernst Lippe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.           
 *                                                                            
 * This library is distributed in the hope that it will be useful,             
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Version $Id: gimppreviewprogressupdater.c,v 1.2 2003/06/11 11:09:12 el Exp $
 */

#include "gimppreviewprogressupdater.h"
#include "libgimp/gimp.h"

#ifndef lint
static char vcid[] G_GNUC_UNUSED =
  "$Id: gimppreviewprogressupdater.c,v 1.2 2003/06/11 11:09:12 el Exp $";
#endif /* lint */

/**
 * gimp_preview_progress_updater_init_for_gimp:
 * @updater: A #GimpPreviewProgressUpdater.
 * 
 * Initialize the GimpPreviewProgressUpdater to use the GIMP
 * for showing the progress.
 **/
void
gimp_preview_progress_updater_init_for_gimp (GimpPreviewProgressUpdater *
                                             updater)
{
  updater->to_preview = FALSE;
}

/**
 * gimp_preview_progress_updater_init_for_preview:
 * @updater: A #GimpPreviewProgressUpdater.
 * @preview: The #GimpPreview.
 * 
 * 
 * Initialize the GimpPreviewProgressUpdater to use the @preview
 * for showing the progress.
 **/
void
gimp_preview_progress_updater_init_for_preview (GimpPreviewProgressUpdater *
                                                updater,
                                                GimpPreview * preview)
{
  updater->to_preview = TRUE;
  updater->preview = preview;
  gimp_preview_progress_updater_set_fraction (updater, 0.0);
}

/**
 * gimp_preview_progress_updater_set_fraction:
 * @updater: A #GimpPreviewProgressUpdater.
 * @fraction: The fraction that has been completed.
 * This must be a value in the range 0 to 1.
 * 
 * Update the progress bar to @fraction completed.
 * 
 * Return value: Returns FALSE when the rendering progress
 * should halt.
 **/
gboolean
gimp_preview_progress_updater_set_fraction (GimpPreviewProgressUpdater *
                                            updater, gdouble fraction)
{
  if (updater->to_preview)
    {
      return (gimp_preview_progress_set_fraction (updater->preview,
                                                  fraction));

    }
  else
    {
      return (gimp_progress_update (fraction));
    }
}
