/* Gimppreview test bed plug-in
 * Copyright (C) 2003 Ernst Lippe
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Version $Id: gimppreviewtest.c,v 1.2 2003/06/11 11:09:14 el Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <libgimp/gimp.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include "gimppreview.h"
#include "gimppreviewprogressupdater.h"
#include "gimppreviewutil.h"

#ifndef lint
static char vcid[] G_GNUC_UNUSED =
  "$Id: gimppreviewtest.c,v 1.2 2003/06/11 11:09:14 el Exp $";
#endif /* lint */

GimpDrawable *drawable;
gint run_flag;

/* Declare local functions. */
static void query (void);
static void run (char *name,
                 gint nparams,
                 GimpParam * param,
                 gint * nreturn_vals, GimpParam ** return_vals);
static gint dialog ();
static void doit (void);
static void check_config (void);
static void progress_bar_toggled (GtkToggleButton * button, gpointer data);
static void zoom_controls_toggled (GtkToggleButton * button, gpointer data);
static void scrols_bars_toggled (GtkToggleButton * button, gpointer data);
static void padding_changed_handler (GtkSpinButton * spinbutton,
                                     gpointer data);
static void show_original_toggled (GtkToggleButton * button, gpointer data);
static void draw_button_toggled (GtkToggleButton * button, gpointer data);
static void render_callback (GtkWidget * widget,
                             GimpPreviewUpdateEvent * event, gpointer data);

typedef enum
{
  DRAW_NONE,
  DRAW_UNSCALED_DRAWABLE,
  DRAW_SCALED_DRAWABLE,
  DRAW_UNSCALED_ROW,
  DRAW_SCALED_ROW
}
RenderDrawingType;

GimpPlugInInfo PLUG_IN_INFO = {
  NULL,                         /* init_proc */
  NULL,                         /* quit_proc */
  query,                        /* query_proc */
  run,                          /* run_proc */
};


typedef struct
{
  RenderDrawingType drawing_type;
}
config;

const config default_config = {
  DRAW_NONE,
};

config my_config;

struct
{
  GtkWidget *ok_button;
  GtkSpinButton *xpad_spinbutton, *ypad_spinbutton;
}
my_widgets;




MAIN ()
     static void query ()
{
  static GimpParamDef args[] = {
    {GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive"},
    {GIMP_PDB_IMAGE, "image", "Input image (unused)"},
    {GIMP_PDB_DRAWABLE, "drawable", "Input drawable"},
  };
  static GimpParamDef *return_vals = NULL;
  static gint nargs = (gint) (sizeof (args) / sizeof (args[0]));
  static gint nreturn_vals = 0;

  gimp_install_procedure ("plug_in_gimppreview_test",
                          "Test bed for GimpPreview widget",
                          "",
                          "Ernst Lippe",
                          "Ernst Lippe",
                          "2003",
                          "<Image>/Filters/Generic/Preview test ...",
                          "RGB*, GRAY*",
                          GIMP_PLUGIN,
                          nargs, nreturn_vals, args, return_vals);
}

static void
run (char *name, gint n_params, GimpParam * param,
     gint * nreturn_vals, GimpParam ** return_vals)
{
  static GimpParam values[1];
  GimpRunMode run_mode;
  GimpPDBStatusType status = GIMP_PDB_SUCCESS;

  (void) name;                  /* Shut up warnings about unused parameters. */
  *nreturn_vals = 1;
  *return_vals = values;

  run_mode = param[0].data.d_int32;

  /*  Get the specified drawable  */
  drawable = gimp_drawable_get (param[2].data.d_drawable);

  my_config = default_config;

  if (run_mode == GIMP_RUN_NONINTERACTIVE)
    {
      if (n_params != 3)
        status = GIMP_PDB_CALLING_ERROR;
      else
        {
        }
    }
  else
    {
#if 0
      gimp_get_data ("plug_in_gimppreview_test", &my_config);
#endif

      if (run_mode == GIMP_RUN_INTERACTIVE)
        {
          /* Oh boy. We get to do a dialog box, because we can't really expect the
           * user to set us up with the right values using gdb.
           */
          check_config ();
          if (!dialog ())
            {
              /* The dialog was closed, or something similarly evil happened. */
              status = GIMP_PDB_EXECUTION_ERROR;
            }
        }
    }

  if (status == GIMP_PDB_SUCCESS)
    {

      /*  Make sure that the drawable is gray or RGB color  */
      if (gimp_drawable_is_rgb (drawable->drawable_id) ||
          gimp_drawable_is_gray (drawable->drawable_id))
        {
          doit ();

          if (run_mode != GIMP_RUN_NONINTERACTIVE)
            gimp_displays_flush ();
          if (run_mode == GIMP_RUN_INTERACTIVE)
            gimp_set_data ("plug_in_gimppreview_test", &my_config,
                           sizeof (my_config));
        }
      else
        {
          status = GIMP_PDB_EXECUTION_ERROR;
        }
      gimp_drawable_detach (drawable);
    }

  values[0].type = GIMP_PDB_STATUS;
  values[0].data.d_status = status;
}

/***************************************************
 * GUI stuff
 */

static void
close_callback (GtkWidget * widget, gpointer data)
{
  (void) widget;                /* Shut up warnings about unused parameters. */
  (void) data;
  gtk_main_quit ();
}

static void
ok_callback (GtkWidget * widget, gpointer data)
{
  (void) widget;                /* Shut up warnings about unused parameters. */
  run_flag = 1;
  gtk_widget_destroy (GTK_WIDGET (data));
}


/* Checks that the configuration is valid for the image type */
static void
check_config (void)
{
}


static gint
dialog ()
{
  GtkWidget *dlg;
  GtkWidget *button;
  GtkWidget *preview;
  GtkWidget *vpaned;
  GtkWidget *hpaned;
  GtkWidget *empty_box;
  GtkWidget *table;
  GtkWidget *entry_label;
  GtkWidget *spin_button;
  GtkWidget *toggle_button;
  GtkWidget *radio_button;
  gchar **argv;

  gint argc;

  argc = 1;
  argv = g_new (gchar *, 1);
  argv[0] = g_strdup ("previewtest");

  gtk_init (&argc, &argv);
  gtk_rc_parse (gimp_gtkrc ());

  dlg = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dlg), "Preview Test");
  gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_MOUSE);
  g_signal_connect (G_OBJECT (dlg), "destroy",
                    G_CALLBACK (close_callback), NULL);

  /*  Action area  */
  my_widgets.ok_button = button = gtk_button_new_with_label ("OK");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  g_signal_connect (G_OBJECT (button), "clicked",
                    G_CALLBACK (ok_callback), G_OBJECT (dlg));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->action_area), button, TRUE,
                      TRUE, 0);
  gtk_widget_grab_default (button);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("Cancel");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  g_signal_connect_object (G_OBJECT (button), "clicked",
                           G_CALLBACK (close_callback), G_OBJECT (dlg), 0);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->action_area), button, TRUE,
                      TRUE, 0);
  gtk_widget_show (button);

  hpaned = gtk_hpaned_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), hpaned, TRUE, TRUE,
                      0);

  vpaned = gtk_vpaned_new ();
  gtk_paned_pack1 (GTK_PANED (hpaned), vpaned, TRUE, TRUE);

  preview =
    gimp_preview_new_with_args (drawable, 0, 0, 1.0, TRUE, TRUE, TRUE);
  g_signal_connect (G_OBJECT (preview), "update-preview",
                    G_CALLBACK (render_callback), drawable);

  gtk_paned_pack1 (GTK_PANED (vpaned), preview, TRUE, TRUE);
  empty_box = gtk_hbox_new (TRUE, 0);
  gtk_paned_pack2 (GTK_PANED (vpaned), empty_box, TRUE, TRUE);

  gtk_widget_show (preview);
  gtk_widget_show (empty_box);
  gtk_widget_show (vpaned);
  gtk_widget_show (hpaned);

  table = gtk_table_new (8, 2, FALSE);
  gtk_paned_pack2 (GTK_PANED (hpaned), table, TRUE, TRUE);

  entry_label = gtk_label_new ("x");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 0, 1);
  gtk_widget_show (entry_label);
  spin_button =
    gtk_spin_button_new (gimp_preview_get_x_adjustment
                         (GIMP_PREVIEW (preview)), 1.0, 0);
  gtk_table_attach_defaults (GTK_TABLE (table), spin_button, 1, 2, 0, 1);
  gtk_widget_show (spin_button);

  entry_label = gtk_label_new ("y");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 1, 2);
  gtk_widget_show (entry_label);
  spin_button =
    gtk_spin_button_new (gimp_preview_get_y_adjustment
                         (GIMP_PREVIEW (preview)), 1.0, 0);
  gtk_table_attach_defaults (GTK_TABLE (table), spin_button, 1, 2, 1, 2);
  gtk_widget_show (spin_button);

  entry_label = gtk_label_new ("scale");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 2, 3);
  gtk_widget_show (entry_label);
  spin_button =
    gtk_spin_button_new (gimp_preview_get_scale_adjustment
                         (GIMP_PREVIEW (preview)), 0.1, 10);
  gtk_table_attach_defaults (GTK_TABLE (table), spin_button, 1, 2, 2, 3);
  gtk_widget_show (spin_button);

  entry_label = gtk_label_new ("zoom controls");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 3, 4);
  gtk_widget_show (entry_label);
  toggle_button = gtk_toggle_button_new ();
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_button), TRUE);
  gtk_table_attach (GTK_TABLE (table), toggle_button, 1, 2, 3, 4, 0, 0, 0, 0);
  g_signal_connect (G_OBJECT (toggle_button), "toggled",
                    G_CALLBACK (zoom_controls_toggled), preview);
  gtk_widget_show (toggle_button);

  entry_label = gtk_label_new ("progress bar");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 4, 5);
  gtk_widget_show (entry_label);
  toggle_button = gtk_toggle_button_new ();
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_button), TRUE);
  gtk_table_attach (GTK_TABLE (table), toggle_button, 1, 2, 4, 5, 0, 0, 0, 0);
  g_signal_connect (G_OBJECT (toggle_button), "toggled",
                    G_CALLBACK (progress_bar_toggled), preview);
  gtk_widget_show (toggle_button);

  entry_label = gtk_label_new ("scroll bars");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 5, 6);
  gtk_widget_show (entry_label);
  toggle_button = gtk_toggle_button_new ();
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_button), TRUE);
  gtk_table_attach (GTK_TABLE (table), toggle_button, 1, 2, 5, 6, 0, 0, 0, 0);
  g_signal_connect (G_OBJECT (toggle_button), "toggled",
                    G_CALLBACK (scrols_bars_toggled), preview);
  gtk_widget_show (toggle_button);

  entry_label = gtk_label_new ("xpad");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 6, 7);
  gtk_widget_show (entry_label);
  spin_button = gtk_spin_button_new_with_range (0, 100, 1);
  my_widgets.xpad_spinbutton = GTK_SPIN_BUTTON (spin_button);
  gtk_table_attach_defaults (GTK_TABLE (table), spin_button, 1, 2, 6, 7);
  g_signal_connect (G_OBJECT (spin_button), "value-changed",
                    G_CALLBACK (padding_changed_handler), preview);
  gtk_widget_show (spin_button);

  entry_label = gtk_label_new ("ypad");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 7, 8);
  gtk_widget_show (entry_label);
  spin_button = gtk_spin_button_new_with_range (0, 100, 1);
  my_widgets.ypad_spinbutton = GTK_SPIN_BUTTON (spin_button);
  gtk_table_attach_defaults (GTK_TABLE (table), spin_button, 1, 2, 7, 8);
  g_signal_connect (G_OBJECT (spin_button), "value-changed",
                    G_CALLBACK (padding_changed_handler), preview);
  gtk_widget_show (spin_button);

  entry_label = gtk_label_new ("show original");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 8, 9);
  gtk_widget_show (entry_label);
  toggle_button = gtk_toggle_button_new ();
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_button), TRUE);
  gtk_table_attach (GTK_TABLE (table), toggle_button, 1, 2, 8, 9, 0, 0, 0, 0);
  g_signal_connect (G_OBJECT (toggle_button), "toggled",
                    G_CALLBACK (show_original_toggled), preview);
  gtk_widget_show (toggle_button);

  entry_label = gtk_label_new ("no draw");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 9, 10);
  gtk_widget_show (entry_label);
  radio_button = gtk_radio_button_new (NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), TRUE);
  gtk_table_attach (GTK_TABLE (table), radio_button, 1, 2, 9, 10, 0, 0, 0, 0);
  g_signal_connect (G_OBJECT (radio_button), "toggled",
                    G_CALLBACK (draw_button_toggled),
                    GINT_TO_POINTER (DRAW_NONE));
  gtk_widget_show (radio_button);

  entry_label = gtk_label_new ("unscaled drawable");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 10, 11);
  gtk_widget_show (entry_label);
  radio_button =
    gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_button));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), FALSE);
  gtk_table_attach (GTK_TABLE (table), radio_button, 1, 2, 10, 11, 0, 0, 0,
                    0);
  g_signal_connect (G_OBJECT (radio_button), "toggled",
                    G_CALLBACK (draw_button_toggled),
                    GINT_TO_POINTER (DRAW_UNSCALED_DRAWABLE));
  gtk_widget_show (radio_button);

  entry_label = gtk_label_new ("scaled drawable");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 11, 12);
  gtk_widget_show (entry_label);
  radio_button =
    gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_button));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), FALSE);
  gtk_table_attach (GTK_TABLE (table), radio_button, 1, 2, 11, 12, 0, 0, 0,
                    0);
  g_signal_connect (G_OBJECT (radio_button), "toggled",
                    G_CALLBACK (draw_button_toggled),
                    GINT_TO_POINTER (DRAW_SCALED_DRAWABLE));
  gtk_widget_show (radio_button);

  entry_label = gtk_label_new ("unscaled row");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 12, 13);
  gtk_widget_show (entry_label);
  radio_button =
    gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_button));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), FALSE);
  gtk_table_attach (GTK_TABLE (table), radio_button, 1, 2, 12, 13, 0, 0, 0,
                    0);
  g_signal_connect (G_OBJECT (radio_button), "toggled",
                    G_CALLBACK (draw_button_toggled),
                    GINT_TO_POINTER (DRAW_UNSCALED_ROW));
  gtk_widget_show (radio_button);

  entry_label = gtk_label_new ("scaled row");
  gtk_table_attach_defaults (GTK_TABLE (table), entry_label, 0, 1, 13, 14);
  gtk_widget_show (entry_label);
  radio_button =
    gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON (radio_button));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), FALSE);
  gtk_table_attach (GTK_TABLE (table), radio_button, 1, 2, 13, 14, 0, 0, 0,
                    0);
  g_signal_connect (G_OBJECT (radio_button), "toggled",
                    G_CALLBACK (draw_button_toggled),
                    GINT_TO_POINTER (DRAW_SCALED_ROW));
  gtk_widget_show (radio_button);

  gtk_widget_show (table);
  gtk_widget_show (dlg);
  gimp_preview_update (GIMP_PREVIEW (preview));
  gtk_main ();
  gdk_flush ();
  return run_flag;
}

static void
progress_bar_toggled (GtkToggleButton * button, gpointer data)
{
  GimpPreview *preview = GIMP_PREVIEW (data);

  gimp_preview_set_progress_bar_visible (preview,
                                         gtk_toggle_button_get_active
                                         (button));
}

static void
zoom_controls_toggled (GtkToggleButton * button, gpointer data)
{
  GimpPreview *preview = GIMP_PREVIEW (data);

  gimp_preview_set_zoom_controls_visible (preview,
                                          gtk_toggle_button_get_active
                                          (button));
}

static void
scrols_bars_toggled (GtkToggleButton * button, gpointer data)
{
  GimpPreview *preview = GIMP_PREVIEW (data);

  gimp_preview_set_scroll_bars_visible (preview,
                                        gtk_toggle_button_get_active
                                        (button));
}

static void
show_original_toggled (GtkToggleButton * button, gpointer data)
{
  GimpPreview *preview = GIMP_PREVIEW (data);

  gimp_preview_set_show_original (preview,
                                  gtk_toggle_button_get_active (button));
}

static void
draw_button_toggled (GtkToggleButton * button, gpointer data)
{
  my_config.drawing_type = (RenderDrawingType) data;
}

static void
padding_changed_handler (GtkSpinButton * spinbutton, gpointer data)
{
  GimpPreview *preview = GIMP_PREVIEW (data);
  gtk_misc_set_padding (GTK_MISC (preview->image),
                        gtk_spin_button_get_value (my_widgets.
                                                   xpad_spinbutton),
                        gtk_spin_button_get_value (my_widgets.
                                                   ypad_spinbutton));
}

void
doit (void)
{
}

static void
render_callback (GtkWidget * widget, GimpPreviewUpdateEvent * event,
                 gpointer data)
{
  GimpPreview *preview = GIMP_PREVIEW (widget);
  GimpDrawable *drawable = (GimpDrawable *) data;
  GimpDrawable *dest_drawable;
  GimpPreviewProgressUpdater preview_progress_updater;

  dest_drawable =
    gimp_preview_get_temp_drawable (drawable, event->image_x, event->image_y,
                                    event->image_width, event->image_height);

  gimp_preview_progress_updater_init_for_preview (&preview_progress_updater,
                                                  preview);
  gimp_preview_progress_updater_set_fraction (&preview_progress_updater, 0.0);

  switch (my_config.drawing_type)
    {
    case DRAW_NONE:
      break;
    case DRAW_UNSCALED_DRAWABLE:
      gimp_preview_draw_unscaled_drawable (preview, dest_drawable);
      break;
    case DRAW_SCALED_DRAWABLE:
      {
        const gint32 dest_image_id =
          gimp_drawable_image (dest_drawable->drawable_id);
        GimpDrawable *scaled_drawable;

        /* This is a crude hack because the standard GIMP scaling algorithm
           is different from that of the preview */

        /* Get rid of the extra size that was added by gimp_preview_get_temp_drawable(). */
        gimp_image_crop (dest_image_id, event->image_width,
                         event->image_height, 0, 0);
        gimp_image_scale (dest_image_id, event->preview_width,
                          event->preview_height);
        scaled_drawable = gimp_drawable_get (dest_drawable->drawable_id);

        gimp_preview_draw_scaled_drawable (preview, scaled_drawable);
      }
      break;
    case DRAW_UNSCALED_ROW:
      {
        GimpPixelRgn pixel_region;
        guchar *row_buf = g_new (guchar, 4 * event->image_width);
        gint row_nr;
        gboolean keep_on_going = TRUE;

        gimp_pixel_rgn_init (&pixel_region, dest_drawable,
                             0, 0,
                             event->image_width, event->image_height, FALSE,
                             FALSE);
        for (row_nr = 0; keep_on_going && (row_nr < event->image_height);
             row_nr++)
          {
            gimp_pixel_rgn_get_row (&pixel_region, row_buf, 0, row_nr,
                                    event->image_width);
            gimp_preview_draw_unscaled_row (preview,
                                            gimp_drawable_type
                                            (dest_drawable->drawable_id),
                                            row_nr, event->image_width,
                                            row_buf);
            keep_on_going =
              gimp_preview_progress_updater_set_fraction
              (&preview_progress_updater,
               row_nr / (gdouble) event->image_height);

          }
        g_free (row_buf);
      }
      break;
    case DRAW_SCALED_ROW:
      {
        const gint32 dest_image_id =
          gimp_drawable_image (dest_drawable->drawable_id);
        GimpDrawable *scaled_drawable;

        GimpPixelRgn pixel_region;
        guchar *row_buf = g_new (guchar, 4 * event->preview_width);
        gint row_nr;
        gboolean keep_on_going = TRUE;

        /* This is a crude hack because the standard GIMP scaling algorithm
           is different from that of the preview */

        /* Get rid of the extra size that was added by gimp_preview_get_temp_drawable(). */
        gimp_image_crop (dest_image_id, event->image_width,
                         event->image_height, 0, 0);

        gimp_image_scale (dest_image_id, event->preview_width,
                          event->preview_height);
        scaled_drawable = gimp_drawable_get (dest_drawable->drawable_id);

        gimp_pixel_rgn_init (&pixel_region, scaled_drawable,
                             0, 0,
                             event->preview_width, event->preview_height,
                             FALSE, FALSE);
        for (row_nr = 0; keep_on_going && (row_nr < event->preview_height);
             row_nr++)
          {
            gimp_pixel_rgn_get_row (&pixel_region, row_buf, 0, row_nr,
                                    event->preview_width);
            gimp_preview_draw_scaled_row (preview,
                                          gimp_drawable_type
                                          (scaled_drawable->drawable_id),
                                          row_nr, event->preview_width,
                                          row_buf);
            keep_on_going =
              gimp_preview_progress_updater_set_fraction
              (&preview_progress_updater,
               row_nr / (gdouble) event->preview_height);

          }
      }
      break;
    }
  gimp_preview_progress_updater_set_fraction (&preview_progress_updater, 1.0);
  gimp_preview_free_temp_drawable (dest_drawable);
}
